#!/usr/bin/perl
# 11/18/2011 Liz Marchani
# To execute this perl script, enter the commands "perl plotIBD.pl <worksheet>"
# where <worksheet> is a parameter file. See README.plotIBD for more details on <worksheet>. 
# This assumes MORGAN v3.0.3 and IBDgraph 2.0, available from 
# http://www.stat.washington.edu/thompson/Genepi/pangaea.shtml

################################
### Read in parameter values ###
################################
$worksheet = $ARGV[0];
open (TEMPLATE,"$worksheet")||die("can't open the worksheet $worksheet.\n");
while (<TEMPLATE>) {
	chomp;
	if ($_ =~ m/^chrname =/) { # Name of the chromosome of interest
		($tag, $chrname) = split("=", $_);
		$chrname =~ s/^\s+//; $chrname =~ s/\s+$//; $chrname =~ s/^"//;	$chrname =~ s/"$//; 
	i}
	elsif ($_ =~ m/^pedsize =/) { # Number of subjects in the pedigree
		($tag, $pedsize) = split("=", $_);
		$pedsize =~ s/^\s+//; $pedsize =~ s/\s+$//; $pedsize =~ s/^"//; $pedsize =~ s/"$//;
	}
	elsif ($_ =~ m/^indstring =/) { # Identify subjects to draw
		($tag, $indstring) = split("=", $_);
		$indstring =~ s/^\s+//; $indstring =~ s/\s+$//; $indstring =~ s/^"//; $indstring =~ s/"$//;
		(@inds) = split(" ", $indstring);
		$ninds = @inds;
		$longid = 0;
		for ($ii = 0; $ii < $ninds; $ii++) {
		        if (length($inds[$ii]) > $longid) {$longid = length($inds[$ii]);}
	  	}
		$idlength = $longid + 2; $idformat = '%-'; $idformat = $idformat."$idlength"."s";
	}
	elsif ($_ =~ m/^realizations =/) { # The path to the gl_auto FGL output file
		($tag, $realizations) = split("=", $_);
		$realizations =~ s/^\s+//; $realizations =~ s/\s+$//; $realizations =~ s/^"//; $realizations =~ s/"$//;
	}
        elsif ($_ =~ m/^whichfig =/) { # Draw a single realization (1) or single locus (2)
		($tag, $whichfig) = split("=", $_);
		$whichfig =~ s/^\s+//; $whichfig =~ s/\s+$//; $whichfig =~ s/^"//; $whichfig =~ s/"$//;
	}
	elsif ($_ =~ m/^mapfile =/) { # The path to the marker map file
		($tag, $mapfile) = split("=", $_);
		$mapfile =~ s/^\s+//; $mapfile =~ s/\s+$//; $mapfile =~ s/^"//; $mapfile =~ s/"$//;
	}
	elsif ($_ =~ m/^chrlength =/) { # The total modeled length of the chromosome of interest
		($tag, $chrlength) = split("=", $_);
		$chrlength =~ s/^\s+//; $chrlength =~ s/\s+$//; $chrlength =~ s/^"//; $chrlength =~ s/"$//;
	}
	elsif ($_ =~ m/^whichreal =/) { # The single realization to draw
		($tag, $whichreal) = split("=", $_);
		$whichreal =~ s/^\s+//; $whichreal =~ s/\s+$//; $whichreal =~ s/^"//; $whichreal =~ s/"$//;
	}
	elsif ($_ =~ m/^markname =/) { # The single locus to draw
		($tag, $markname) = split("=", $_);
		($mymark, $endmark) = split (":", $markname);
		$mymark =~ s/^\s+//; $mymark =~ s/\s+$//; $mymark =~ s/^"//; $mymark =~ s/"$//;
	}
        elsif ($_ =~ m/^classfile =/) { # The path to the IBD_graph output equivalence class file
		($tag, $classfile) = split("=", $_);
		$classfile =~ s/^\s+//; $classfile =~ s/\s+$//; $classfile =~ s/^"//; $classfile =~ s/"$//;
	}
} 

###################################
### Founder Genome Label Colors ###
###################################
#This thirty-six element array holds the RGB names of the colors to be used
@colors = ("#696969", "#3B14AF", "#FF7400", "#009999", "#CD0074", "#9FEE00", "#9F3ED5", "#FFFF00", "#4671D5", "#FFAA00", "#00CC00", "#FF0000", "#D3D3D3", "#886ED7", "#FFB273", "#5CCCCC", "#E667AF", "#C9F76F", "#AD66D5", "#FFFF73", "#6C8CD5", "#FFD073", "#67E667", "#FF7373", "black", "#210372", "#A64B00", "#006363", "#85004B", "#679B00", "#48036F", "#A6A600", "#2A4480", "#A66F00", "#008500", "#A60000");
@colornames = ("grey1", "indigo1", "sunset1", "aqua1", "wine1", "lime1", "violet1", "lemon1", "blue1", "orange1", "green1", "red1", "grey2", "indigo2", "sunset2", "aqua2", "wine2", "lime2", "violet2", "lemon2", "blue2", "orange2", "green2", "red2", "grey3", "indigo3", "sunset3", "aqua3", "wine3", "lime3","violet3", "lemon3", "blue3", "orange3", "green3", "red3");
##########################################
### Execute the appropriate subroutine ###
##########################################
if ($whichfig == 1) { single_realization(); }
elsif ($whichfig == 2) { single_locus(); }
else {print "Bad value for whichfig. Choose 1 to draw a single realization or 2 to draw a single locus.\n";}
#############################################
### Draw all loci at a single realization ###
#############################################
sub single_realization {
	##############################
	### Read in the marker map ###
	##############################
	open (MAP,"$mapfile")||die ("Can't open marker map file $mapfile\n");
	@positions = ();
	while (<MAP>){
		chomp;
		(@positions)=split(" ",$_); # the ith element of this array has the map position of the (i-1)th marker
	}
	close(MAP);
	#make sure your segment lengths are evenly spaced
	$longpos = 0; #longest position has this many characters
	$longest = 0; #the 'longest' element of @positions (1-nmarkers, not zero indexed)
	$longdec = 0; #longest precision
	$nmarkers = @positions; #number of markers
	for ($ii = 0; $ii <= $nmarkers; $ii++) {
		$current = $positions[$ii];
		if (length($current) > $longpos) {$longpos = length($current);}
		$longest = $ii;
		($before,$after) = split('\.', $positions[$ii]);
		if (length($after) > $longdec) {$longdec = length($after);}
	}
	($before, $after) = split('\.', $positions[$longest-1]);
	if (length($after) == $longdec) {$poslength = $longpos + 2;}
	else { $poslength = $longpos + ($longdec-length($after)) +2;}
	$posformat = '%-';
	$posformat = $posformat."$poslength".".$longdec"."f"; # Floating precision string
	#####################
	### Grab the data ###
	#####################
	$. = 0; #initialize the line counter
	$maternal = ''; $paternal = ''; #initialize the strings that hold segment information
	$old_id = ''; #initialize the string to keep track of subjects within the pedigree within a realization
	@labels = (); #initialize the array of arrays that holds founder genome labels (FGLs) for each member of @inds
	%allFGLs = (); #initialize the hash of all unique FGLs used by @inds
	open (DATA, "$realizations")||die("Can't open FGL realization file $realizations\n");
	while (<DATA>){
		if ($. < (($whichreal-1)*2*$pedsize+1) || $. > $whichreal*2*$pedsize){ next; } #skip to desired iteration
		$id = ''; $qtl = ''; $start = ''; $number = ''; @pairs = (); #initialize
		
                $id = ''; $qtl = ''; $start = ''; $number = ''; @pairs = (); #initialize
		($id,$qtl,$start,$number,@pairs)=split(" ",$_);
  		$wanted = 0; $whichind = '';
		for ($ii=0; $ii < $ninds; $ii++) { #check if subject is in @inds
			if ($id eq $inds[$ii]) { $wanted ++; $whichind = $ii; }
		}
		if ($wanted == 0) { next; } #skip subjects not in @inds
		if ($id ne $old_id) { # Need to close the old and open the new individual
			############################################
			### Make a gnuplot data file for $old_id ###
			############################################
			open (GNUDATA, ">$old_id.dat")||die("Can't create data file for gnuplot $old_id.dat\n");
			#The file needs one column for each maternal and paternal segment,
			#so that if the maternal chromosome has 3 segments and the paternal chromosome
			#has 2 segments, GNUDATA will have 6 columns: id, maternal1, maternal2, maternal3,
			#paternal1, paternal2; values will equal the length of the segment in Haldane cM.
			(@matdata) = split(" ", $maternal);
			$matlength = 0; #initialize
			$matlength = @matdata;
			$mchunks = $matlength - 1; #number of IBD segments on maternal chromosome
			(@patdata) = split(" ", $paternal);
			$patlength = 0; #initialize
			$patlength = @patdata;
			$pchunks = $patlength - 1; #number of IBD segments on paternal chromosome
			$mpad = ''; #Add 0's for the paternal columns in the maternal string
			for ($ii = 0; $ii < $pchunks; $ii++) {$mpad = $mpad.sprintf("$posformat", 0);}
			$maternal = $maternal.$mpad;
			$ppad = ''; #Add 0's for the maternal columns in the paternal string
			for ($ii = 0; $ii < $mchunks; $ii++) {$ppad = $ppad.sprintf("$posformat", 0);}
			$fatpat = '';
			$fatpat = $fatpat.substr($paternal,0,$idlength);
			$fatpat = $fatpat.$ppad;
			$fatpat = $fatpat.substr($paternal,$idlength);
			printf GNUDATA "$maternal\n$fatpat";
			close(GNUDATA);
			###########################################################
			### Collect the maternal chromosome information for $id ###
			###########################################################
			$maternal = ''; #initialize string describing maternal chromosome
			$maternal = $maternal.sprintf("$idformat", "M" );
			if ($number==0) { #all marker loci are inherited IBD from the same founder
				$maternal = $maternal.sprintf("$posformat", $chrlength);
				push(@{$labels[$whichind]}, "$start");
				$allFGLs{$start}++;
			}
			else { #navigate across $number of switches
				for ($ii = 0; $ii <= $number; $ii++) { #for each segment
					if ($ii == 0) { #the first segment
						$segment = 0; #initialize length of this segment
						$segment = $positions[$pairs[$ii*2]-1] - 0;
						$maternal = $maternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$start");
						$allFGLs{$start}++;
					}
					elsif ($ii == $number) { #the last segment
						$segment = 0; #initialize segment length
						$segment = $chrlength - $positions[$pairs[($ii*2)-2]-1];
						$maternal = $maternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$pairs[($ii*2)-1]");
						$allFGLs{$pairs[($ii*2)-1]}++;
					}
					else { #a middle segement
						$segment = 0; #initialize segment length
						$segment = $positions[$pairs[$ii*2]-1] - $positions[$pairs[($ii*2)-2]-1];
						$maternal = $maternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$pairs[($ii*2)-1]");
						$allFGLs{$pairs[($ii*2)-1]}++;
					}
				}
			}
			$old_id = $id;
		}
		elsif ($id eq $old_id){ #still collecting data on $id
			###########################################################
		  	### Collect the paternal chromosome information for $id ###
			###########################################################
			$paternal = ''; #initialize
			$paternal = $paternal.sprintf("$idformat", "P");
			if ($number==0) { # All marker loci are inherited IBD from the same founder
				$paternal = $paternal.sprintf("$posformat", $chrlength);
				push(@{$labels[$whichind]}, "$start");
				$allFGLs{$start}++;
			}
			else {  # Navigate across $number of switches
				for ($ii = 0; $ii <= $number; $ii++) {
					if ($ii == 0) { #the first segment
						$segment = 0; #initialize
						$segment = $positions[$pairs[$ii*2]-1] - 0;
						$paternal = $paternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$start");
						$allFGLs{$start}++;
					}
					elsif ($ii == $number) { #the last segment length
						$segment = 0; #initialize
						$segment = $chrlength - $positions[$pairs[($ii*2)-2]-1];
						$paternal = $paternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$pairs[($ii*2)-1]");
						$allFGLs{$pairs[($ii*2)-1]}++;
					}
					else { #a middle segment
						$segment = 0; #initialize
						$segment = $positions[$pairs[$ii*2]-1] - $positions[$pairs[($ii*2)-2]-1];
						$paternal = $paternal.sprintf("$posformat", $segment);
						push(@{$labels[$whichind]}, "$pairs[($ii*2)-1]");
						$allFGLs{$pairs[($ii*2)-1]}++;
					}
				}
			}
			$old_id = $id;
		}
	}
	close(DATA);
	###############################################################
	### Make a gnuplot data file for the last subject collected ###
	###############################################################
	open (GNUDATA, ">$old_id.dat")||die("Can't create data file for gnuplot $old_id.dat\n");
	(@matdata) = split(" ", $maternal); #count the IBD segments on the maternal chromosome
	$matlength = 0; #initialize
	$matlength = @matdata;
	$mchunks = $matlength - 1;
	(@patdata) = split(" ", $paternal); #count the IBD segments on the paternal chromosome
	$patlength = 0;
	$patlength = @patdata;
	$pchunks = $patlength - 1;
	$mpad = ''; #add 0's for the paternal columns in the maternal string
	for ($ii = 0; $ii < $pchunks; $ii++) {$mpad = $mpad.sprintf("$posformat", 0);}
	$maternal = $maternal.$mpad;
	$ppad = ''; #add 0's for the maternal columns in the paternal string
	for ($ii = 0; $ii < $mchunks; $ii++) {$ppad = $ppad.sprintf("$posformat", 0);}
	$fatpat = '';
	$fatpat = $fatpat.substr($paternal,0,$idlength);
	$fatpat = $fatpat.$ppad;
	$fatpat = $fatpat.substr($paternal,$idlength);
	printf GNUDATA "$maternal\n$fatpat";
	close(GNUDATA);
	@uniqueFGLs = keys %allFGLs; #find the FGLs represented in @inds and make sure you have enough colors for them
	@sortedFGLs = sort { $a <=> $b } @uniqueFGLs;
	$nFGLs = @sortedFGLs;
	$ncolors = @colors;
	if ($nFGLs > $ncolors) {
		die("There are more founder genome labels to be represented than colors to represent them.\n");
	}
	%colorkey=(); #initialize
	for ($ii = 0; $ii < $nFGLs; $ii++) { #map each FGL to a unique color
		$colorkey{$sortedFGLs[$ii]} = $colors[$ii];
	}
	#################################################################
	### Create the file of gnuplot commands to draw the color key ###
	#################################################################
	$keystring = '';
	$keystring = $keystring.sprintf("set term postscript portrait font ".'"Times-Roman, 14"'."\n");
	$keystring = $keystring.sprintf("set output ".'"key.ps'."\n");
	$keystring = $keystring.sprintf("set nokey\n");
	$keystring = $keystring.sprintf("set title ".'"Color Key" font "Times-Roman, 20"'."\n");
	$keystring = $keystring.sprintf("set style fill solid 1.00 border -1\n");
	$keystring = $keystring.sprintf("set boxwidth 0.75\n");
	$keystring = $keystring.sprintf("set style histogram rowstacked\n");
	$keystring = $keystring.sprintf("set style data histograms\n");
	$keystring = $keystring.sprintf("set yrange [0:$nFGLs]\n");
	$dataones = ''; #key.dat is a string of space-delimited 1's, the number = $ninds
	$ytics = ''; #this string will hold the title and position of each y tic
	$holdplot = ''; #this string will hold the series of "plot" statements
	$column = 0; # this keeps track of which column of key.dat to plot. arbitrary, but it works.
	for ($ii = 0; $ii < $nFGLs; $ii++) {
	  	$dataones = $dataones.sprintf("1 ");
   		$tic = $ii +0.5;
  		$column = $ii +1;
		$ytics = $ytics.sprintf('"'."$colornames[$ii]".'"'." $tic, ");
		$holdplot = $holdplot.sprintf(' "key.dat"'." u $column lc rgb ".'"'."$colors[$ii]".'"'.", ");
	}
	chop $ytics; chop $ytics; chop $holdplot; chop $holdplot;
	open (KEYDATA, ">key.dat")||die("Can't create the key data file.\n");
	printf(KEYDATA $dataones);
	close(KEYDATA);
	$keystring = $keystring.sprintf("set ytics ($ytics)\n");
	$keystring = $keystring.sprintf("set grid ytics\n");
	$keystring = $keystring.sprintf("set noxtics\n");
	$keystring = $keystring.sprintf("plot $holdplot\n");
	open (GNUKEY, ">key.gnuplot")||die("Can't create gnuplot command file for the color key.\n");
	printf(GNUKEY $keystring);
	close(GNUKEY);
	#########################################################################
	### Create the file of gnuplot commands to draw the chromosome figure ###
	#########################################################################
	$gnustring = '';
	$gnustring = $gnustring.sprintf("set term postscript landscape ".'"Times-Roman, 11"'."\n");
	$gnustring = $gnustring.sprintf("set output ".'"'."chr$chrname".'.ps"'."\n"); #name that file after the chromosome
	$gnustring = $gnustring.sprintf("set nokey\n"); #Don't print an inaccurate figure legend
	$gnustring = $gnustring.sprintf("set ylabel ".'"'."Marker Number".'"'." 1, 0 font ".'"'. "Times-Roman, 14".'"'."\n"); #label the y axis
	$gnustring = $gnustring.sprintf("set yrange [0:$chrlength]\n"); #set the range of the y axis
	$gnustring = $gnustring.sprintf("set noxtics\n");
	$gnustring = $gnustring.sprintf("set xlabel ".'" "'." 0.0,-1.5\n");
	$gnustring = $gnustring.sprintf("set bmargin 4\n");
	$ytics = '';
	for ($ii = 0; $ii < $nmarkers; $ii++) {
		$tag = $ii +1;
		$ytics = $ytics.sprintf('"'."$tag".'"'." $positions[$ii], ");
	}
	chop $ytics; chop $ytics;
	$gnustring = $gnustring.sprintf("set ytics ($ytics)\n");
	$gnustring = $gnustring.sprintf("set style fill solid 1.00 border -1\n"); #fill in the blocks of the histogram
	$gnustring = $gnustring.sprintf("set style histogram rowstacked\n"); #draw a stacked histogram
	$gnustring = $gnustring.sprintf("set style data histograms\n"); #the data is in the format for histograms
	$gnustring = $gnustring.sprintf("set ytics nomirror\n"); #do not put ytics on the opposite axis
	$gnustring = $gnustring.sprintf("set grid y\n"); #draw dashed lines from each ytic
	$gnustring = $gnustring.sprintf("set boxwidth 0.95\n"); #allow for a small gap between maternal and paternal bars
	$gnustring = $gnustring.sprintf("set title ".'"'."Chromosome $chrname Realization $whichreal".'"'." font ".'"Times-Roman, 20"'."\n");
	$gnustring = $gnustring.sprintf("plot "); #start drawing the figure
	for ($ii = 0; $ii < $ninds; $ii++) { #one new histogram per element of @inds
		$gnustring = $gnustring.sprintf("newhistogram ".'"'."$inds[$ii]".'"');
		$col = 2;
		$nchunks = @{$labels[$ii]};
		for ($jj = 0; $jj < $nchunks; $jj++) {
			$gnustring = $gnustring.sprintf (", ".'"'."$inds[$ii]".'.dat"'." u $col lc rgb ".'"'."$colorkey{$labels[$ii][$jj]}".'"');
			$col++;
		}
		$gnustring = $gnustring.sprintf(", ");
	}
	chop $gnustring; chop $gnustring;
	open (GNUCHR, ">chromosome$chrname.gnuplot")||die("Can't create gnuplot command file for chromsome figure.\n");
	printf(GNUCHR $gnustring);
	close(GNUCHR);
}
######################################################
### Draw all equivalence classes at a single locus ###
######################################################
sub single_locus {
	#########################################
	### Summarize the equivalence classes ###
	#########################################
	open (CLASSES, "$classfile")||die("Can't open equivalence class file $classfile.\n");
	$. = 0; #initialize the line counter
	$count = 0; #initialize the count weight of the equivalence class
	$representative = 0; #initialize the representative realization of the equivalence class
	%classes = (); # keys are representative realizations and values are counts of realizations per class
	$nclasses = 0; #initialize the counter of equivalence classes
	while (<CLASSES>) {
		if ($. == 1) {next;}
		elsif ($. == 2) {
			(@words) = split(" ", $_);
			$nreal= $words[1]; # $nreal is the total number of FGL graphs analyzed
		}
		elsif ($. == 3) { next; } #skip the 2nd line of the equivalence class file
		else {
			($count, $tmp) = split(":", $_);
			$count =~ s/^\s+//; $count =~ s/\s+$//;	chomp $count;
			($representative, @others) = split(",", $tmp);
			chomp $representative;
			$representative =~ s/^\s+//; $representative =~ s/\s+$//;
			$classes{$representative} = $count;
			$nclasses++;
		}
	}
	close(CLASSES);
	################################################################
	### Sort the equivalence classes by count (descending order) ###
	### and then by representative realization (ascending order) ###
	################################################################
	@allkeys = (); @sortedkeys = (); @sortedvalues = (); #initialize
	@allkeys = keys %classes; #collect all key values from the classes hash
	@sortedkeys = sort { $a <=> $b } @allkeys;
	@sortedvalues = sort { $classes{$b} <=> $classes{$a} } @sortedkeys;
	$padding = ''; #these zeros allow for the maternal and paternal chromosomes to be drawn beside each other.
	for ($ii = 0; $ii < $nclasses; $ii++) { $padding = $padding."0 "; }
	open (PLOTCLASS, ">classes.dat")||die("Can't open the gnuplot data file for class weights classes.dat.\n");
	foreach $element (@sortedvalues) { #print once for the maternal chromosome
		printf PLOTCLASS "$classes{$element} "; }
	printf PLOTCLASS "$padding\n$padding";
	foreach $element (@sortedvalues) { #print again for the paternal chromosome
		printf PLOTCLASS "$classes{$element} "; }
	close(PLOTCLASS);
	#################################################
	### Comb through the gl_auto FGL realizations ###
	#################################################
	$. = 0; #initialize the line counter
	$old_id = ''; # initialize the id of the previously recorded FGL
	$holding = ''; # initialize the string with form: $element maternal_FGL paternal_FGL
	%allFGLs = (); # this hash will hold all FGLs carried by the members of @inds across all classes
	open (DATA, "$realizations")||die("Can't open FGL realization file $realizations\n");
	while (<DATA>){
		foreach $element (@sortedkeys) { #using @sortedkeys so a single loop through the data file will be sufficient
			if ($. < (($element-1)*2*$pedsize+1) || $. > $element*2*$pedsize){next;}
			($id,$qtl,$start,$number,@pairs)=split(" ",$_);
			$wanted = 0;#initialize the indicator variable that $id is a member of @inds
			$whichind = ''; # initialize the variable storing which element of @inds matches $id
			for ($ii=0; $ii < $ninds; $ii++) { #check if subject is in @inds
				if ($id eq $inds[$ii]) { $wanted ++; $whichind = $ii; }
			}
			if ($wanted == 0) {next;} #skip subjects not in @inds
#			print "element is $element. line $.. $_\n";
			if ($number == 0) { #there are no crossover events on this chromosome
				$holding = $holding."$start ";
				$allFGLs{$start}++; }
			else { #loop through the crossovers on this chromosome
				$tempFGL = $start; #the FGL associated with the previous marker
				for ($jj = 0; $jj < $number; $jj++) { 
					if ($pairs[$jj*2] > $mymark) { #this crossover happens AFTER my marker
						$holding = $holding."$tempFGL ";
						$allFGLs{$tempFGL}++; 
						$jj = $number; #end the loop
					}
					else { $tempFGL = $pairs[($jj*2)+1]; }
					if ($jj == ($number -1)) { #this is the last switch and it is before $mymark
						$holding = $holding."$tempFGL ";
						$allFGLs{$tempFGL}++; 
						$jj == $number;
					}
				}
			}
			if ($old_id eq $id) { # this is the 2nd time you've seen this id
				open (INDDATA, ">>$id.dat")||die("Can't create individual data file $id.dat.\n");
				print INDDATA "$element $holding\n";
				close(INDDATA);
				$holding = '';
				$old_id = '';
			}
			else {$old_id = $id;}
		}
	}
	close(DATA);
	##################################################################
	### Map the unique FGLs represented in @inds to a unique color ###
	##################################################################
	@uniqueFGLs = (); @sortedFGLs = (); $nFGLs = 0; $ncolors = 0; #initialize	
	@uniqueFGLs = keys %allFGLs;
	@sortedFGLs = sort { $a <=> $b } @uniqueFGLs;
	$nFGLs = @sortedFGLs; # this is the number of unique FGLs to be drawn
	$ncolors = @colors;
	if ($nFGLs > $ncolors) { die("Too few colors are defined to represent all founder genome labels.\n"); }
	%colorkey = ();
	for ($ii = 0; $ii < $nFGLs; $ii++) { $colorkey{$sortedFGLs[$ii]} = $colors[$ii]; }
       	#################################################################
	### Create the file of gnuplot commands to draw the color key ###
	#################################################################
  	$keystring = '';
	$keystring = $keystring."set term postscript portrait font ".'"Times-Roman, 14"'."\n";
	$keystring = $keystring."set output ".'"key.ps'."\n";
	$keystring = $keystring."set nokey\n";
	$keystring = $keystring."set title ".'"Color Key" font "Times-Roman, 20"'."\n";
	$keystring = $keystring."set style fill solid 1.00 border -1\n";
	$keystring = $keystring."set boxwidth 0.75\n";
	$keystring = $keystring."set style histogram rowstacked\n";
	$keystring = $keystring."set style data histograms\n";
	$keystring = $keystring."set yrange [0:$nFGLs]\n";
	$dataones = ''; #key.dat is a string of space-delimited 1's, the number = $ninds
	$ytics = ''; #this string will hold the title and position of each y tic
	$holdplot = ''; #this string will hold the series of "plot" statements
	$column = 0; # this keeps track of which column of key.dat to plot.
	for ($ii = 0; $ii < $nFGLs; $ii++) {
		$dataones = $dataones."1 ";
		$tic = $ii +0.5;
		$column = $ii +1;
		$ytics = $ytics.'"'."$colornames[$ii]".'"'." $tic, ";
		$holdplot = $holdplot.' "key.dat"'." u $column lc rgb ".'"'."$colors[$ii]".'"'.", ";
	}
	chop $ytics; chop $ytics; chop $holdplot; chop $holdplot; #delete trailing spaces and commas
	open (KEYDATA, ">key.dat")||die("Can't create the key data file.\n");
	printf(KEYDATA $dataones);
	close(KEYDATA);
	$keystring = $keystring."set ytics ($ytics)\n";
	$keystring = $keystring."set grid ytics\n";
	$keystring = $keystring."set noxtics\n";
	$keystring = $keystring."plot $holdplot\n";
	open (GNUKEY, ">key.gnuplot")||die("Can't create gnuplot command file for the color key.\n");
	printf(GNUKEY $keystring);
	close(GNUKEY);
	################################################################################
	### Create the file of gnuplot commands to draw the equivalence class figure ###
	################################################################################
	$gnustring = '';
	$gnustring = $gnustring.'set term postscript landscape "Times-Roman" 11'."\n";
	$gnustring = $gnustring."set title ".'"'."IBD sharing at chromosome $chrname marker(s) $markname".'"'.' font "Times-Roman, 20"'."\n";
	$gnustring = $gnustring."set output ".'"classes.ps"'."\n";
	$gnustring = $gnustring."set tics out nomirror\n";
	$gnustring = $gnustring."set tmargin 5\n";
	$gnustring = $gnustring."set bmargin 5\n";
	$gnustring = $gnustring."set nokey\n";
	$gnustring = $gnustring."set noxtics\n";
	$gnustring = $gnustring."set xlabel ".'" "'." 0.0,-1.5\n";
	$gnustring = $gnustring."set ylabel ".'"Equivalence Class Distribution"'." font ".'"'."Times-Roman, 14".'"'."\n";
	$classstring = "#Class Name, Example realization, Count of classmates\n";
	$ytics = '';
	$yvalue = 0;
	for ($ii = 0; $ii < $nclasses; $ii++) {
		$plusone = $ii +1;
		$yvalue = $yvalue + $classes{$sortedvalues[$ii]};
		$ytics = $ytics.'"Class '."$plusone".'"'." $yvalue, ";
		$classstring = $classstring."Class $plusone, $sortedvalues[$ii], $classes{$sortedvalues[$ii]}\n";
	}
	chomp $classstring; chop $ytics; chop $ytics; #delete trailing whitespace, commas, and newline
	open (CLASSKEY, ">class_descriptions.txt")||die("Can't create a summary of equivalence classes class_descriptions.txt.\n");
	printf(CLASSKEY $classstring);
	close(CLASSKEY);
	$gnustring = $gnustring."set ytics ($ytics)\n";
	$gnustring = $gnustring."set style fill solid 1.00 border -1\n";
	$gnustring = $gnustring."set style histogram rowstacked\n";
	$gnustring = $gnustring."set style data histograms\n";
	$gnustring = $gnustring."set grid y\n";
	$gnustring = $gnustring."set boxwidth 0.95\n";
	$gnustring = $gnustring."plot "; #start drawing the figure one
       	for ($ii = 0; $ii < $ninds; $ii++) { #new histogram per element of @inds
		open (IND, "$inds[$ii].dat")||die("Can't open an individual's data file $inds[$ii].dat.\n");
		%mom; #this hash will hold maternal FGLs across equivalence classes
		%dad; #this hash will hold paternal FGLs across equivalence classes
		while(<IND>) {
			($representative, $momfgl, $dadfgl) = split();
			$mom{$representative} = $momfgl;
			$dad{$representative} = $dadfgl;
		}
		$gnustring = $gnustring."newhistogram ".'"'."$inds[$ii]".'"';
		$col = 1; #start with the first column
		foreach $rep (@sortedvalues) { #plot maternal data
			$gnustring = $gnustring.", ".'"classes.dat"'." u $col lc rgb ".'"'."$colorkey{$mom{$rep}}".'"';
			$col++;
		}
		foreach $rep (@sortedvalues) { #plot paternal data
			$gnustring = $gnustring.", ".'"classes.dat"'." u $col lc rgb ".'"'."$colorkey{$dad{$rep}}".'"';
			$col++;
	    	}
		$gnustring = $gnustring.sprintf(", ");
	}
	chop $gnustring; chop $gnustring; #delete trailing whitespace and comma
	open (GNUCHR, ">classes.gnuplot")||die("Can't create gnuplot command file for equivalence classes figure.\n");
	printf(GNUCHR $gnustring);
	close(GNUCHR);
}
